# vault

![vault](images/vault.png "vault")<!-- .element width="30%" -->

**By HashiCorp**


## Installation

* téléchargement d'un binaire

  * [https://releases.hashicorp.com/vault/](https://releases.hashicorp.com/vault/)
  * décompresser dans /usr/local/bin
  * configurer les permissions
  * serveur
    * créer un service systemd
  * cli
    * `/usr/local/bin/vault `


## Systemd 

```shell
setcap CAP_IPC_LOCK=+ep /usr/local/bin/vault
setcap CAP_NET_BIND_SERVICE=+eip /usr/local/bin/vault
```

```ini
[Unit]
Description=a tool for managing secrets
Documentation=https://vaultproject.io/docs/
After=network.target
ConditionFileNotEmpty=/etc/vault/vault.hcl

[Service]
User=vault
Group=vault
ExecStart=/usr/local/bin/vault server -config=/etc/vault/vault.hcl
ExecReload=/usr/local/bin/kill --signal HUP $MAINPID

[Install]
WantedBy=multi-user.target
```


## Configuration

`/etc/vault/vault.hcl`

```
backend "file" {
  path = "/var/lib/vault"
}
ui = true
disable_mlock = true
listener "tcp" {
  address     = "10.0.0.1:443"
  tls_cert_file = "/etc/certs/vault.crt"
  tls_key_file  =  "/etc/certs/vault.key"
  tls_disable   = 0
}
```


## Initialisation

```shell
export VAULT_ADDR=https://10.0.0.1
export VAULT_SKIP_VERIFY=True
vault operator init -key-shares=3 -key-threshold=2
```

```shell
Unseal Key 1: 6gAO3lmAhIaHzDAdkK256g2B2Dpeqy+z4jqQCJBID3d8
Unseal Key 2: TO7DDRQXSPC3IQylPEBPjPGAGAGMzjEkjT/FL62m7UUd
Unseal Key 3: CYfEbt83jYsQFcSErHT4Y5NCsrEtfFUE6tjQZKfP632K

Initial Root Token: s.78MykQO2b5qcy03rtoNwmhr1

Vault initialized with 3 key shares and a key threshold of 2. Please securely
distribute the key shares printed above. When the Vault is re-sealed,
restarted, or stopped, you must supply at least 2 of these keys to unseal it
before it can start servicing requests.

Vault does not store the generated master key. Without at least 2 key to
reconstruct the master key, Vault will remain permanently sealed!

It is possible to generate new unseal keys, provided you have a quorum of
existing unseal keys shares. See "vault operator rekey" for more information.
```


## [<i class="fa fa-book" aria-hidden="true"></i> Secrets engines](https://www.vaultproject.io/docs/secrets/)

!["secrets engines"](images/vault-secrets-engines.png "secrets engines")

<!--
* [<i class="fa fa-book" aria-hidden="true"></i> Secrets Engines - getting started](https://learn.hashicorp.com/vault/getting-started/dynamic-secrets)
* [<i class="fa fa-book" aria-hidden="true"></i> AWS Secrets Engine](https://www.vaultproject.io/docs/secrets/aws/index.html)
* [<i class="fa fa-book" aria-hidden="true"></i> Active Directory Secrets Engine](https://www.vaultproject.io/docs/secrets/aws/index.html)
* [<i class="fa fa-book" aria-hidden="true"></i> SSH Secrets Engine](https://www.vaultproject.io/docs/secrets/ssh/index.html)
* [<i class="fa fa-book" aria-hidden="true"></i> KV Secrets Engine](https://www.vaultproject.io/docs/secrets/kv/index.html)
-->


## KV

```shell
$ vault kv get cri/test
====== Data ======
Key          Value
---          -----
password1    secret$

$ vault kv put cri/test password2=secret!
Success! Data written to: cri/test

$ vault kv get cri/test
====== Data ======
Key          Value
---          -----
password2    secret!
```


## KV2

```shell
vault secrets enable -path=cri kv
vault kv enable-versioning cri/ # kv2
```

* les secrets sont versionés
* il est possible d'utiliser PATCH et pas seulement PUT

```shell
$ vault kv patch cri/test password1=secret$
Success! Data written to: cri/test

$ vault kv get cri/test
====== Data ======
Key          Value
---          -----
password1    secret$
password2    secret!
```


## Authentification

!["authentification"](images/vault-auth.png "authentification")


## Authentification

```shell
vault login token=<root-token>
```

* par token
  * root
  * d'application

```shell
$ vault login -method=ldap username=vimazeno
```

* par ldap
  * en réalité génère un token dans ~/.vault-token contenant


## LDAP

```shell
$ vault write auth/ldap/config \
    url="ldaps://samantha.local.isima.fr" \
    userattr="sAMAccountName" \
    userdn="dc=local,dc=isima,dc=fr" \
    groupattr="cn" \
    groupfilter="(&(objectClass=group)(member:1.2.840.113556.1.4.1941:={{.UserDN}}))" \
    groupdn="ou=GROUPES_LOCAUX,dc=local,dc=isima,dc=fr" \
    binddn="cn=vault,ou=Comptes de Services,dc=local,dc=isima,dc=fr" \
    bindpass="secret" \
    insecure_tls="false" \
    starttls="true"
```

[<i class="fa fa-book" aria-hidden="true"></i> LDAP Auth Method](https://www.vaultproject.io/docs/auth/ldap.html)


## Policy

/etc/vault/users/cri.hcl

```
# Write and manage secrets in key-value secret engine
path "secret/*" {
  capabilities = ["create", "read", "update", "delete", "list", "sudo"]
}

# To enable secret engines
path "sys/mounts/*" {
  capabilities = [ "create", "read", "update", "delete" ]
}

path "cubbyhole/*" {
  capabilities = ["create", "read", "update", "delete", "list"]
}

```

```shell
$ vault policy write cri /etc/vault/cri.hcl
```


## appliquer une policy à un groupe ldap

```shell
$ vault write auth/ldap/groups/cri policies=cri
```


## Utilisation

* [binaire à télécharger](https://releases.hashicorp.com/vault/)
  * cross plateform
  * deux variables d'environnement
    * $VAULT_ADDR=https://vault.isima.fr
    * $VAULT_TOKEN ou authentification ldap
ou
* l'[<i class="fa fa-book" aria-hidden="true"></i> api](https://www.vaultproject.io/api/overview)


## Workflow

```shell
$ vault secrets list
$ vault kv list cri/
$ vault kv get cri/services/vault/tokens
$ vault kv get cri/services/vault/tokens # à chaque put on écrase les entrées qu'on ne réécrit pas
$ vault kv get -format=json cri/services/vault/tokens
$ vault kv get -format=json cri/services/vault/tokens | jq .data
$ vault kv get -format=json cri/services/vault/tokens | jq .data.data.root
$ vault kv put cri/test password2=$(date | sha256sum | cut -c -50)
$ vault kv patch cri/test password1=$(date | sha256sum | cut -c -50)
$ vault delete cri/test
```


## création de token

my.hcl

```
path "secret/data/cri/apps/my" {
  capabilities = ["create", "read", "update", "delete", "list"]
}
```

```shell
$ vault policy write vault/hcl/apps/my.hcl
$ vault token create -policy=my
```


## vault/ci/cd

### en local

authentification ldap

### <i class="fa fa-gitlab" aria-hidden="true"></i> CI / CD

![vault CI](images/vault-ci.png)


## bin/setup

```bash
command -v "vault" >/dev/null 2>&1 || {
  echo >&2 "I require vault to run see stack"
  exit 1
}
if [[ -z "${VAULT_ADDR}" ]] ; then
  export VAULT_ADDR=https://vault.isima.fr
fi
if [[ -z "${VAULT_TOKEN}" ]] ; then
  if [[ -z "${VAULT_USERNAME}" ]] ; then
    echo uca username
    read username
    export VAULT_USERNAME=${username}
  fi
  vault login -method=ldap username=$VAULT_USERNAME > /dev/null
  echo " export VAULT_TOKEN=$(cat ~/.vault-token)"
else
  vault login token=${VAULT_TOKEN} > /dev/null
fi
```
<!-- .element style="width: 100%;" -->


## bin/configure

```bash
# lecture des clés vault avec python: la sortie est une liste python UTF8 (u'value')
KV=$(vault read cri/my -format=json | python -c "import sys, json; print json.load(sys.stdin)['data'].keys()")
# converison de la liste python en liste bash
VAULT_KEYS=( $(echo ${KV} | sed -r "s/', u'/' '/g" | sed -r "s/\[u'/'/g" | sed -r "s/\]//g") )
# copie du ttemplatye de configuration en fichier de configuration
cp config.sample.py config.py
# itération sur les clés vault
for i in "${VAULT_KEYS[@]}"
do
  # enlève le permier '
  i=${i%\'}
  # enlève le dernier '
  i=${i#\'}
  sed -i "s|$i|$(vault read cri/my -format=json | jq -r .data.$i | sed -r "s/\n//g")|g" config.py 2>/dev/null
done
```
<!-- .element style="width: 100%;" -->
